<?php
/*
Plugin Name: InsiteLogin
Plugin URI: http://www.eurotraining.it
Description: A plugin that insert the standard login procedure into a page
Version: 0.4
Author: Franco Traversaro
Author URI: mailto:franco.traversaro@eurotraining.it
*/
define('INSITELOGIN_LOGIN_PAGE', 0);
define('INSITELOGIN_LOGGED_IN', 'You are logged in. <a href="%%logout%%">Log out</a>.');
define('INSITELOGIN_LOGGED_IN_TITLE', 'Logout');
define('INSITELOGIN_LOGGED_OUT_TITLE', 'Login');

class InSiteLogin_plugin {
	public $login_page = INSITELOGIN_LOGIN_PAGE;
	public $logged_in = INSITELOGIN_LOGGED_IN;
	public $logged_in_title = INSITELOGIN_LOGGED_IN_TITLE;
	public $logged_out_title = INSITELOGIN_LOGGED_OUT_TITLE;
	public $notice = '';

	public function __construct() {
		$this->manage_options();
		add_action('admin_menu', array($this, 'admin_menu'));
		add_action('admin_head', array($this, 'admin_head'));
		if ($this->login_page) {
			add_action('init', array($this, 'a_init'));
			add_filter('posts_results', array($this, 'f_posts_results'));
			add_filter('get_pages', array($this, 'f_get_pages'), 10, 2);
			add_filter('wp_redirect', array($this, 'f_wp_redirect'), 10, 2);
			add_filter('login_redirect', array($this, 'f_login_redirect'), 10, 2);
		}
		register_activation_hook(__FILE__, array($this, 'activate_plugin'));
		register_deactivation_hook(__FILE__, array($this, 'deactivate_plugin'));
	}

	public function activate_plugin() {
		add_option('insitelogin_login_page', INSITELOGIN_LOGIN_PAGE);
		add_option('insitelogin_logged_in', INSITELOGIN_LOGGED_IN);
		add_option('insitelogin_logged_in_title', INSITELOGIN_LOGGED_IN_TITLE);
		add_option('insitelogin_logged_out_title', INSITELOGIN_LOGGED_OUT_TITLE);
	}

	public function deactivate_plugin() {
		delete_option('insitelogin_login_page');
		delete_option('insitelogin_logged_in');
		delete_option('insitelogin_logged_in_title');
		delete_option('insitelogin_logged_out_title');
	}

	public function a_init() {
		if (empty($_POST) and false!==strpos($_SERVER['PHP_SELF'], 'wp-login.php')) {
			$proto = ( is_ssl() ) ? 'https://' : 'http://';
			$url = parse_url($proto . $_SERVER['SERVER_NAME'] . $_SERVER['PHP_SELF'] . '?' . $_SERVER['QUERY_STRING']);
			if (isset($url['query'])) {
				$separator = (get_option('permalink_structure')) ? '?' : '&';
				wp_redirect(get_permalink($this->login_page) . $separator . $url['query']);
				die();
			} else {
				wp_redirect(get_permalink($this->login_page));
				die();
			}
		}
	}

	public function f_wp_redirect($location, $status) {
		if (preg_match('/^wp-login.php(?:\?(.*))?/', $location, $matches)) {
			$separator = (get_option('permalink_structure')) ? '?' : '&';
			return get_permalink($this->login_page) . $separator . $matches[1];
		} else {
			return $location;
		}
	}

	public function f_login_redirect($default, $requested) {
		if (isset($_REQUEST['redirect_to']) and $_REQUEST['redirect_to']) {
			return $_REQUEST['redirect_to'];
		} else {
			if ($default==admin_url()) {
				return get_permalink($this->login_page);
			} else {
				return $default;
			}
		}
	}

	public function admin_head() {
		echo '<link rel="stylesheet" type="text/css" href="'.WP_PLUGIN_URL. '/insitelogin/style.css">';
	}


	public function manage_options() {
		if (isset($_POST['insitelogin_changeoptions']) and $_POST['insitelogin_changeoptions']==md5(__FILE__)) {
			update_option('insitelogin_login_page', stripslashes($_POST['insitelogin']['login_page']));
			update_option('insitelogin_logged_in', stripslashes($_POST['insitelogin']['logged_in']));
			update_option('insitelogin_logged_in_title', stripslashes($_POST['insitelogin']['logged_in_title']));
			update_option('insitelogin_logged_out_title', stripslashes($_POST['insitelogin']['logged_out_title']));
			$this->notice = '<br /><div class="updated fade" id="message" style="background-color: rgb(255, 251, 204);"><p>Options correctly changed!</p></div>';
		}
		$this->login_page = get_option('insitelogin_login_page');
		$this->logged_in = get_option('insitelogin_logged_in');
		$this->logged_in_title = get_option('insitelogin_logged_in_title');
		$this->logged_out_title = get_option('insitelogin_logged_out_title');

		if (!$this->logged_out_title) $this->logged_out_title=INSITELOGIN_LOGGED_OUT_TITLE;
		if (!$this->logged_in_title) $this->logged_in_title=INSITELOGIN_LOGGED_IN_TITLE;
		if (!$this->logged_in) $this->logged_in=INSITELOGIN_LOGGED_IN;
	}

	public function f_posts_results($posts) {
		if (is_admin() or empty($posts)) {
			return $posts;
		} else {
			foreach ($posts as $key=>$val) {
				if ($val->ID==$this->login_page) {
					$val->post_content = $this->the_plugin();
					$val->post_title = (is_user_logged_in()) ? $this->logged_in_title : $this->logged_out_title;
				}
				$return[ $key ] = $val;
			}
			return $return;
		}
	}

	public function f_get_pages($posts, $r) {
		if (is_admin() or empty($posts)) {
			return $posts;
		} else {
			foreach ($posts as $key=>$val) {
				if ($val->ID==$this->login_page) {
					$val->post_title = (is_user_logged_in()) ? $this->logged_in_title : $this->logged_out_title;
				}
				$return[ $key ] = $val;
			}
			return $return;
		}
	}

	public function the_plugin() {
		global $post;
		$action = isset($_REQUEST['action']) ? $_REQUEST['action'] : '';
		$activated_actions = array('logout', 'lostpassword', 'retrievepassword', 'resetpass', 'rp', 'register');
		if ( in_array($action, $activated_actions) or !is_user_logged_in() ) {
			ob_start();
			include(ABSPATH.'/wp-login.php');
			preg_match('/\<(body)\b(?:.*?)(?:(?:\/))?\>(?:(.+?)\<\/\1\>)?/si', ob_get_clean(), $matches);
			$chroot = $matches[2];
			preg_match_all('/(?:href|action)=("|\')(?:(.*?)\1)/i', $chroot, $matches);
			if (empty($matches)) {
				return $chroot;
			} else {
				$search = $replace = array();
				foreach ($matches[2] as $victim) {
// 					maialata perchè non sono capace a mettere la regola nel preg_match_all
					if (false!==strpos($victim, 'wp-login.php')) {
						$url = parse_url($victim);
						$search[] = $victim;
						if (isset($url['query'])) {
							$separator = (get_option('permalink_structure')) ? '?' : '&';
							$replace[] = get_permalink($this->login_page) . $separator . $url['query'];
						} else {
							$replace[] = get_permalink($this->login_page);
						}
					}
				}
				return str_replace($search, $replace, $chroot);
			}
		} else {
			$separator = (get_option('permalink_structure')) ? '?' : '&';
			$logout = get_permalink($this->login_page) . $separator . 'action=logout';
			return str_ireplace('%%logout%%', $logout, $this->logged_in);
		}
	}

	public function admin_menu() {
		add_options_page(__('InsiteLogin'), __('InsiteLogin'), 10, __FILE__, array($this, 'opzioni'));
	}

	public function opzioni() {
		global $wpdb;

		echo '<div class="wrap">';
		echo '<h2>InsiteLogin options</h2>';
		echo $this->notice;
		echo '<form method="post">';
		echo '<table class="form-table" id="insitelogin_options">';
		echo '<tbody>';

		echo "\n<tr>";
		echo "\n\t".'<td><label for="insitelogin_login_page"><strong>Login page.</strong><br />Original content page will be ignored.</label></td>';
		echo "\n\t".'<td><select id="insitelogin_login_page" name="insitelogin[login_page]" class="insitelogin_width">';
		$pagine = $wpdb->get_results("SELECT `ID`, `post_title`, `menu_order` FROM `{$wpdb->posts}` WHERE `post_status`='publish' AND `post_type`='page' UNION SELECT 0, '', -1 AS menu_order ORDER BY `menu_order`", ARRAY_A);
		if (!empty($pagine)) foreach ($pagine as $row) {
			$check = ($row['ID']==$this->login_page) ? ' selected="selected"' : '';
			echo '<option value="'.$row['ID'].'"'.$check.'>'.$row['post_title'].'</option>';
		}
		echo '</select></td>';
		echo "\n".'</tr>';


		echo "\n<tr>";
		echo "\n\t".'<td><label for="insitelogin_logged_in"><strong>Logged in page.</strong><br /><code>%%logout%%</code>  will be replaced with logout URL.</label></td>';
		echo "\n\t".'<td><textarea id="insitelogin_logged_in" name="insitelogin[logged_in]" class="insitelogin_width">' . $this->logged_in . '</textarea></td>';
		echo "\n".'</tr>';

		echo "\n<tr>";
		echo "\n\t".'<td><label for="insitelogin_logged_in_title"><strong>Logged in page title.</strong></label></td>';
		echo "\n\t".'<td><input type="text" id="insitelogin_logged_in_title" name="insitelogin[logged_in_title]" value="' . $this->logged_in_title . '" class="insitelogin_width" /></td>';
		echo "\n".'</tr>';

		echo "\n<tr>";
		echo "\n\t".'<td><label for="insitelogin_logged_out_title"><strong>Logged out page title.</strong></label></td>';
		echo "\n\t".'<td><input type="text" id="insitelogin_logged_out_title" name="insitelogin[logged_out_title]" value="' . $this->logged_out_title . '" class="insitelogin_width" /></td>';
		echo "\n".'</tr>';

		echo '</tbody>';
		echo '</table>';
		echo '<p class="submit"><input type="submit" value="Save" class="button" /></p>';
		echo '<input type="hidden" value="'.md5(__FILE__).'" name="insitelogin_changeoptions"/>';
		echo '</form>';
		echo '</div>';
	}

}

if (!isset($InSiteLogin_plugin_singleton)) $InSiteLogin_plugin_singleton = new InSiteLogin_plugin();
?>